#!/usr/bin/env python3

"""Flash Lights Impressively
"""

from blynclight import BlyncLight, BlyncLightNotFound
from itertools import cycle
from collections import deque
from time import sleep
import click


def rotatable_color(intensity: str):
    """
    :param intensity: string hexadecimal number
    :return: collections.deque of integers
    """
    I = (0xFF & int(intensity, 16)) >> 0
    return deque([I, 0, 0])


@click.command()
@click.option("-l", "--light-id", default=0, help="Integer light identfier.")
@click.option(
    "-i", "--interval", default=0.1, help="Float interval in seconds to flash."
)
@click.option("-c", "--count", default=-1, help="Integer count to flash impressively.")
@click.option("-I", "--intensity", default="0xff", help="8-bit color intensity.")
@click.option(
    "-a", "--available", is_flag=True, help="List available BlyncLights and exit."
)
def cli(light_id, interval, count, intensity, available):
    """FLI - Flash Light Impressively

    Cycles the light identified by light_id thru red, green and blue
    with the given intensity count times, pausing for interval
    seconds. It can be stupid annoying. Apologize to your cow-orkers
    for me (not a typo).

    """

    if available:
        BlyncLight.report_available()
        return

    try:
        light = BlyncLight.get_light(light_id)
    except BlyncLightNotFound as not_found:
        print(not_found)
        return

    light.on = 1

    color = rotatable_color(intensity)

    try:
        while count != 0:
            count -= 1
            color.rotate(1)
            light.color = list(color)
            sleep(interval)
    except KeyboardInterrupt:
        pass



#!/usr/bin/env python3

"""BlyncLights Love Rainbows!
"""

import click
import math
from blynclight import BlyncLight, BlyncLightNotFound
from time import sleep
from itertools import cycle


def Spectrum(steps=64, frequency=None, phase=None, center=128, width=127):
    """generator function that returns 'steps' rbg tuples.

        steps: optional integer, default=64
    frequency: optional 3-tuple for rbg frequency, default=(.3,.3,.3)
        phase: optional 3-tuple for rbg phase, default=(0,2,4)
       center: optional integer, default=128
        width: optional integer, default=127

    Returns (r, b, g) where each member is a value between 0 and 255.
    """

    frequency = frequency or (0.3, 0.3, 0.3)
    phase = phase or (0, 2, 4)

    for i in range(steps):
        r = int((math.sin(frequency[0] * i + phase[0]) * width) + center)
        b = int((math.sin(frequency[2] * i + phase[2]) * width) + center)
        g = int((math.sin(frequency[1] * i + phase[1]) * width) + center)
        yield (r, b, g)


@click.command()
@click.option("-l", "--light-id", default=0)
@click.option("-s", "--speed", default=1)
def cli(light_id, speed):
    """BlyncLights Love Rainbows!

    """

    colors = [rgb for rgb in Spectrum(steps=255)]

    try:
        b = BlyncLight.get_light(light_id)
    except BlyncLightNotFound as error:
        print(f"{error}")
        exit(-1)

    interval = speed * 0.1

    b.on = True
    try:
        for color in cycle(colors):
            b.color = color
            sleep(interval)
    except KeyboardInterrupt:
        pass



"""
"""

import requests
import json


class Proxy(object):
    """
    """

    def __init__(self, blynclight_url=None):
        """
        """
        self.baseurl = blynclight_url or "http://localhost:8000/blynclight"

    def _call(self, cmd, light_id=None, value=None):
        """
        """
        args = [self.baseurl]

        if light_id or light_id == 0:
            args.append(light_id)

        args.append(cmd)

        if value:
            args.append(value)

        r = requests.get("/".join(map(str, args)))

        return json.loads(r.content.decode("utf-8"))

    @property
    def lights(self):
        """Returns list of available BlyncLights.
        """
        return self._call("status")

    @property
    def status(self, light_id=None):
        """Returns a dictionary of BlyncLight status.
        """
        return self._call("status", light_id)

    def on(self, light_id):
        """:param light_id: integer

        Turns the light associated with light_id on.
        """
        return self._call("on", light_id)

    def off(self, light_id):
        """:param light_id: integer

        Turns the light associated with light_id off.
        """
        return self._call("off", light_id)

    def color(self, light_id, value):
        """:param light_id: integer
        :param value: color specifier ( tuple of ints or int )

        Changes the color of the light associated with light_id.

        """
        if isinstance(value, tuple):
            # convert tuple to string
            value = "0x" + "".join(["{02x}".format(v) for v in value])

        if isinstance(value, int):
            # convert int to hex string
            value = hex(value)

        return self._call("color", light_id, value)

    def red(self, light_id, value):
        """
        """
        return self._call("red", light_id, value)

    def green(self, light_id, value):
        """
        """
        return self._call("green", light_id, value)

    def blue(self, light_id, value):
        """
        """
        return self._call("blue", light_id, value)

    def flash_on(self, light_id):
        raise NotImplementedError("flash_on")

    def flash_off(self, light_id):
        raise NotImplementedError("flash_on")

    def flash_speed(self, light_id, value):
        """
        """
        raise NotImplementedError("flash_speed")



#!/usr/bin/env python3

from blynclight import BlyncLight
import click
from time import sleep
from pprint import pprint


@click.command()
@click.option("-l", "--light-id", default=0, help="Integer light identifer")
@click.option("-r", "--red", default=0, help="Integer 0-255")
@click.option("-b", "--blue", default=0, help="Integer 0-255")
@click.option("-g", "--green", default=0, help="Integer 0-255")
@click.option("--off/--on", default=True)
@click.option("--bright/--dim", default=False)
@click.option("--flash/--no-flash", default=False)
@click.option("-s", "--speed", default=0, help="0, 1, 2, 4")
@click.option(
    "-v", "--verbose", is_flag=True, default=False, help="Prints the light status."
)
@click.option("-d", "--duration", default=-1)
@click.option(
    "-a", "--available", is_flag=True, default=False, help="Show available lights."
)
def cli(
    light_id, red, blue, green, off, bright, flash, speed, verbose, duration, available
):
    """Initialize the state of a connected BlyncLight and
    activate the light for 'duration' seconds.
    """

    if available:
        for light_id, info in enumerate(BlyncLight.available_lights()):
            # XXX better print of available light info
            print("Light Identifier:", light_id)
            pprint(info)
        return

    light = BlyncLight.get_light(light_id)

    with light.updates_paused():
        light.red = red
        light.blue = blue
        light.green = green
        light.off = off
        light.dim = bright
        light.flash = flash
        light.speed = speed

    if verbose:
        print(light)

    while duration != 0:
        duration -= 1
        sleep(1)


if __name__ == "__main__":
    cli()



"""Human Input Device

This module provides a ctypes interface to the USB hidapi C language
library. Functions exposed for use by python via ctypes are:

- void  hid_init(void)
- void  hid_exit(void)
- void *hid_enumerate(int, int)
- void  hid_free_enumeration(void *)
- void *hid_open(int, int)
- void *hid_open_path(char *)
- void  hid_close(void *)
- uint  hid_write(void *, uchar_t *, size_t)
- uint  hid_read(void *, uchar_t *, size_t)
- uint  hid_read_timeout(void, uchar_t *, size_t, int)


The HidDevice class encapsulates all of these functions (with the
exception of hid_init and hid_exit) and provides a more pythonic
interface e.g.:

> from hid import HidDevice
> all_devices = HidDevice.enumerate()
> first_device = HidDevice.from_dict(all_devices[0])
> d = HidDevice(0xe0e0, 0x0001)
> buf = bytes([0xff, 0, 0, 0xff])
> d.write(buf)
4


The hidapi library setup and teardown is done for the user; hid_init()
is called when the module is imported and hid_exit() is scheduled to be
called on process exit using the atexit module.

"""

import ctypes
import ctypes.util
import atexit

__all__ = ["HidDevice"]


class _DeviceInfo(ctypes.Structure):
    """_DeviceInfo is the return data-type of hidapi.hid_enumerate.
    """

    def as_dict(self) -> dict:
        """Translate DeviceInfo to a dictionary whose keys are the structure's
        field names and the values are the corresponding field values.
        """
        info = {}
        for name, typ in self._fields_:
            if name == "next":
                continue
            info[name] = getattr(self, name, None)
        return info


# the _fields_ array is defined after declaring the class because
# this structure has a 'next' pointer in it that of type DeviceInfo
# and the ctypes.Structure class can't handle self-referencing
# pointers.

_DeviceInfo._fields_ = [
    ("path", ctypes.c_char_p),
    ("vendor_id", ctypes.c_ushort),
    ("product_id", ctypes.c_ushort),
    ("serial_number", ctypes.c_wchar_p),
    ("release_number", ctypes.c_ushort),
    ("manufacturer_string", ctypes.c_wchar_p),
    ("product_string", ctypes.c_wchar_p),
    ("usage_page", ctypes.c_ushort),
    ("usage", ctypes.c_ushort),
    ("interface_number", ctypes.c_int),
    ("next", ctypes.POINTER(_DeviceInfo)),
]


class _DeviceHandle(ctypes.Structure):
    """An opaque handle returned by hidapi.hid_open and used
    as an argument in the functions:
    hidapi.hid_read
    hidapi.hid_read_timeout
    hidapi.hid_write
    """

    pass


_hidapi_libraries = ["hidapi", "hidapi-hidraw", "hidapi-libusb", "hidapi-iohidmanager"]

for stem in _hidapi_libraries:
    fullname = ctypes.util.find_library(stem)
    if fullname:
        hidapi = ctypes.cdll.LoadLibrary(fullname)
        break
else:
    raise ImportError("failed to locate hidapi shared object")

_function_signatures = [
    ("hid_enumerate", [ctypes.c_ushort, ctypes.c_ushort], ctypes.POINTER(_DeviceInfo)),
    ("hid_free_enumeration", [ctypes.POINTER(_DeviceInfo)], None),
    (
        "hid_open",
        [ctypes.c_ushort, ctypes.c_ushort, ctypes.c_wchar_p],
        ctypes.POINTER(_DeviceHandle),
    ),
    ("hid_open_path", [ctypes.c_char_p], ctypes.POINTER(_DeviceHandle)),
    ("hid_close", [ctypes.POINTER(_DeviceHandle)], None),
    (
        "hid_write",
        [ctypes.POINTER(_DeviceHandle), ctypes.POINTER(ctypes.c_byte), ctypes.c_size_t],
        ctypes.c_int,
    ),
    (
        "hid_read",
        [ctypes.POINTER(_DeviceHandle), ctypes.POINTER(ctypes.c_byte), ctypes.c_size_t],
        ctypes.c_int,
    ),
    (
        "hid_read_timeout",
        [
            ctypes.POINTER(_DeviceHandle),
            ctypes.POINTER(ctypes.c_byte),
            ctypes.c_size_t,
            ctypes.c_int,
        ],
        ctypes.c_int,
    ),
]

for fname, argstype, restype in _function_signatures:
    function = getattr(hidapi, fname)
    function.argstype = argstype
    function.restype = restype

hidapi.hid_init()
atexit.register(hidapi.hid_exit)


class HidDevice:
    """A USB Human Input Device (HID) Device

    Provides read and write access to devices that adhere to the USB
    HID protocol. The HidDevice.enumerate class method can be used to
    discover currently attached devices.

    """

    _opened = set()

    @classmethod
    def enumerate(cls, vendor_id: int = 0, product_id: int = 0) -> list:
        """Returns a list of dictionaries describing USB devices.  Specifying
        vendor_id and product_id will only return devices whose
        vendor_id or product_id match the ones given. If no USB devices are
        found, an empty list is returned. See hid._DeviceInfo for more
        information on the contents of the dictionary.

        :param vendor_id: optional integer
        :param product_id: optional integer
        :return: list

        """
        devices = []
        head = hidapi.hid_enumerate(vendor_id, product_id)
        cursor = head
        while cursor:
            devices.append(cursor.contents.as_dict())
            cursor = cursor.contents.next
        hidapi.hid_free_enumeration(head)
        return devices

    @classmethod
    def from_dict(cls, info: dict):
        """Returns a configured hid.HidDevice object from a
        DeviceInfo-derived dictionary.

        :param info: dictionary
        :return: hid.HidDevice
        """

        try:
            return cls(info["vendor_id"], info["product_id"])
        except KeyError:
            raise LookupError("dictionary missing vendor_id or product_id key")

    def __init__(self, vendor_id, product_id, path=None):
        """
        :param vendor_id: 16-bit integer quantity
        :param product_id: 16-bit integer quantity
        :param path: optional string

        Raises ValueError if the vendor_id:product_id pair
        specifies a device that is already open.
        """
        self.vendor_id = vendor_id & 0xFFFF
        self.product_id = product_id & 0xFFFF
        self.path = path
        # XXX this a hack to keep from re-opening a USB device and
        #     inducing a crash in libhidapi.

        if self.identifier in self._opened:
            raise ValueError(f"{self.identifier} already in use")
        self._opened.add(self.identifier)

    def __del__(self):
        """The hid.HidDevice is closed when the object is deleted.
        """
        self.close()

    def __repr__(self):
        return (
            f"{self.__class__.__name__}"
            f"(vendor_id=0x{self.vendor_id:04x},"
            f"product_id=0x{self.product_id:04x},"
            f"path={self.path})"
        )

    @property
    def identifier(self):
        """A string concatenation of the vendor and product identifiers.
        e.g. 0xff11:0x0001
        """
        try:
            return self._identifier
        except AttributeError:
            pass
        self._identifier = f"0x{self.vendor_id:04x}:0x{self.product_id:04x}"
        return self._identifier

    @property
    def handle(self):
        """An opaque hid._DeviceHandle structure pointer used
        to specify the device for I/O operations.
        """
        try:
            return self._handle
        except AttributeError:
            pass
        self._handle = hidapi.hid_open(self.vendor_id, self.product_id, self.path)
        if not self._handle:
            raise LookupError(f"no such device: {self.identifier}")
        self._opened.add(self.identifier)
        return self._handle

    def close(self):
        """Closes the hidapi handle and sets the handle attribute to None.
        """
        # Avoid using the handle getter method here, use _handle
        # instead.  Otherwise we can crash the process with an
        # unintentional SIGSEGV or SIGFAULT while we are shutting
        # things down.
        try:
            if self._handle:
                hidapi.hid_close(self._handle)
        except AttributeError:
            pass
        self._handle = None
        self._opened.discard(self.identifier)

    def write(self, data: bytes) -> int:
        """Write data to the device.

        :param data: bytes buffer of data
        :return: number of bytes written
        """
        return hidapi.hid_write(self.handle, data, len(data))

    def read(self, nbytes: int, timeout: int = None) -> bytes:
        """Reads data from the device into the given bytes buffer.

        :param nbytes: integer
        :param timeout: optional timeout in milliseconds
        :return: bytes read
        """

        buf = (ctypes.c_ubyte * nbytes)()

        if timeout:
            hidapi.hid_read_timeout(self.handle, buf, nbytes, timeout)
        else:
            hidapi.hid_read(self.handle, buf, nbytes)
        return bytearray(buf)



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['blynclight', 'blynclight.hid', 'blynclight.scripts']

package_data = \
{'': ['*']}

install_requires = \
['click>=7.0,<8.0', 'requests>=2.21,<3.0']

entry_points = \
{'console_scripts': ['blync = blynclight.scripts.blync:cli',
                     'fli = blynclight.scripts.fli:cli',
                     'rainbow = blynclight.scripts.rainbow:cli',
                     'throbber = blynclight.scripts.throbber:cli']}

setup_kwargs = {
    'name': 'blynclight',
    'version': '0.4.6',
    'description': 'Python language bindings for Embrava BlyncLight devices.',
    'long_description': 'Embrava BlyncLight\n==================\n\n|pypi| |license| |python|\n\n**blynclight** is a Python 3 package that provides bindings for the\n`Embrava`_ BlyncLight family of products. These bindings have been\ntested on MacOS and Linux using an Embrava V30 USB connected light.\n\n\nInstall\n-------\n\n0. Install `hidapi`_ for your platform:\n\n.. code:: bash\n\n          (rpm Linux distros)# yum install XXXX\n          (apt Linux distros)# apt-get install XXXX\n          (macOS using brew) $ brew install hidapi\n\n1. pip\n\n.. code:: bash\n\n\t  $ pip3 install blynclight\n\n2. Clone the repo\n\n.. code:: bash\n\n\t  $ git clone https://github.com/JnyJny/blynclight.git\n\t  $ cd blynclight; pip3 install -e .\n\n  \nUninstall\n---------\n\n.. code:: bash\n\n\t  $ pip3 uninstall blynclight\n\n\t  \n\nUsage\n-----\n\nOnce installed, the BlyncLight is yours to command!\n\n.. code:: python\n\n\tfrom blynclight import BlyncLight\n\n\tlight = BlyncLight.first_light()\n\n\tred, blue, green = (255, 0, 0), (0, 255, 0), (0, 0, 255)\n\t\n\tlight.color = green           # the light is off and green\n\tlight.on = True               # the light is on and green\n\tlight.flash = True            # the light is on, flashing and green\n\tlight.color = red             # the light is on, flashing and red\n\tlight.flash = False           # the light is on and red\n\tlight.bright = False          # the light is on, dim and red\n\tlight.color = blue            # the light is on, dim and blue\n\tlight.bright = True           # the light is on and blue\n\tlight.on = False              # the light is off and blue\n\t\nMore examples can be found in the contrib directory.\n\n\n.. |pypi| image:: https://img.shields.io/pypi/v/blynclight.svg?style=flat-square&label=version\n    :target: https://pypi.org/pypi/blynclight\n    :alt: Latest version released on PyPi\n\n.. |python| image:: https://img.shields.io/pypi/pyversions/blynclight.svg?style=flat-square\n   :target: https://pypi.org/project/blynclight/\n   :alt: Python Versions\t  \n\n.. |license| image:: https://img.shields.io/badge/license-apache-blue.svg?style=flat-square\n    :target: https://github.com/erikoshaughnessy/blynclight/blob/master/LICENSE\n    :alt: Apache license version 2.0  \n\n.. _Embrava: https://embrava.com\n\n.. _hidapi: https://github.com/signal11/hidapi\n\n\n',
    'author': "Erik O'Shaughnessy",
    'author_email': 'erik.oshaughnessy@gmail.com',
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



#!/usr/bin/env python3

"""BlyncLight intensifies.
"""

import click
from blynclight import BlyncLight, BlyncLightNotFound
from time import sleep
from itertools import cycle


def Gradient(start, stop, step, red=True, green=False, blue=False):
    """
    """
    colors = []
    for i in range(start, stop, step):
        colors.append((i if red else 0, i if blue else 0, i if green else 0))
    return colors


@click.command()
@click.option("-l", "--light-id", type=int, default=0)
@click.option("-r", "--red", is_flag=True, default=False)
@click.option("-g", "--green", is_flag=True, default=False)
@click.option("-b", "--blue", is_flag=True, default=False)
@click.option("-w", "--white", is_flag=True, default=False)
@click.option("-f", "--fast", default=0)
@click.option("-d", "--dim", is_flag=True, default=False)
def cli(light_id, red, green, blue, white, fast, dim):
    """BlyncLight intensifies.
    """

    if white:
        red = True
        green = True
        blue = True

    if not any([red, green, blue]):
        red = True

    step = 8 * (min(fast, 24) + 1)

    colors = Gradient(0, 255, step, red, green, blue)

    colors.extend(c for c in reversed(colors))

    try:
        b = BlyncLight.get_light(light_id)
    except BlyncLightNotFound as not_found:
        print(not_found)
        exit(-1)

    b.on = False
    b.color = (0, 0, 0)
    b.dim = dim

    try:
        b.on = True
        while True:
            for color in cycle(colors):
                b.color = color
                sleep(0.05)
    except KeyboardInterrupt:
        pass
